# WMDRV version 0.5a

**Wacom serial tablet and bus mouse driver**

PC-9801 シリーズの DOS 上で動作する、RS-232C 接続の WACOM 製ペンタブレット及びバスマウスドライバです。
ドライバの制御割込はマイクロソフト仕様に対応しています。

このソフトウェアは3条項 BSD ライセンスの下にリリースされています。

同様のソフトウェアとして [タリスマン](https://www.vector.co.jp/soft/dos/util/se016284.html) が存在しますが、GD シリーズのペンタブレットに対応していないようだったため、GD シリーズに対応したドライバを作成した結果完成したのが本ソフトウェアになります。


## 内容物

- README.md
    - このファイルです。
- LICENSE
    - 本ソフトウェアのライセンスが記載されたファイルです。
- wmdrv.asm, msg.inc
    - 本ソフトウェアのソースコードです。
- wmdrv.com
    - 本ソフトウェアの実行バイナリです。


## 動作環境

このソフトウェアは PC-9801 シリーズの DOS 上で動作するプログラムです。

動作を確認している画面モードは16色400ラインモードのみです。
これ以外の画面モードでは正常に動作しない可能性があります。

現在動作確認済みのワコムのペンタブレットは以下4種類です。
これら以外の機種は非対応です。
また、作者が試した範囲では XD シリーズは常に RS=0 を出力しているようで、フロー制御が必須のインターフェースにそのまま接続した場合、通信ができませんでした。

- UD シリーズ
- KT シリーズ
- GD シリーズ(初代 Intuos)
- XD シリーズ(Intuos 2)

タブレットとの通信には以下6種類の回線が使えます。

- 標準の RS-232C 回線
- PC-9861K 互換の第2回線と第3回線
- 2ndCCU
- I-O DATA 製 RSA-98II 及び RSA-98III の16550互換モード


## プログラムの使い方

プログラム実行時のコマンド書式は `wmdrv [command] (options...)` です。

`[command]` には以下の3通りの値が指定できます。

- `s`: 常駐します
- `r`: 常駐解除します
- `h`: ヘルプを表示します

常駐時には以下6つのオプションを指定できます。

- `/c` オプション
    - タブレットとの通信に利用する RS-232C 回線を指定します
    - 0から4の値で以下5種類の回線を指定できます(デフォルト値は0です)
        - 0: 標準搭載の第1回線
        - 1,2: PC-9861K 互換の第2及び第3回線
        - 3: 2ndCCU
        - 4: RSA-98II 及び RSA-98III の 16550 互換モード
- `/a` オプション
    - RS-232C 回線を制御する際の IO ポートのアドレスを指定します
    - PC-9861K 互換回線を使用する場合 IO アドレスの bit11～8 を指定できます
        - 例えば `/a2` とすると IO ポートとして 02b*h を利用します
        - この場合のデフォルト値は0です
    - RSA-98II 及び RSA-98III を使用する場合 IO ベースアドレスを指定できます
        - 例えば `/a438` とすると 438h～43fh の範囲の IO ポートを利用します
        - この場合のデフォルト値は238hです
- `/i` オプション
    - RS-232C 回線の割込の INT を指定します
        - 4を指定すると INT41 で割り込まれるものとして常駐します
    - RSA-98II 及び RSA-98III 以外の回線を使用する場合は指定しても無視されます
- `/p` オプション
    - 左ボタン押下扱いになる筆圧の最低値を設定します
    - `/p50` であれば筆圧の値が50以上の場合に左ボタンを押下したことになります
    - 初期値は以下の通りです
        - UD, KT シリーズ: 20
        - GD, XD シリーズ: 100
- `/m` オプション
    - タブレットの検知領域の上下左右の余白幅を設定します
    - カンマ区切りで1,2,4何れかの個数の数値を指定します
        - `/mx`: 上下左右すべての余白幅を `x` にします
        - `/mx,y`: (上下,左右)=(`x`,`y`)になるように余白幅を設定します
        - `/mx,y,z,w`: (上,下,左,右)=(`x`,`y`,`z`,`w`)になるように設定します
    - 初期値は機種によらず上下左右すべて80です
- `/k` オプション
    - バスマウスのミッキー値を指定します
    - 初期値は8です
- `/s` オプション
    - タブレットとマウスのカーソル座標を同期させるかを設定します
    - 設定値は以下の通りです
        - 0: 同期させない
        - 非0: 同期させる(初期値)
- `/d` オプション
    - `int 33h (c0h <= ax <= c2h)` の独自ファンクションを使えるようにします


## 常駐時のメッセージ

常駐する際にはタブレットの情報と各種設定値が表示されます。
座標範囲はペンタブから得られるペンの座標の値の範囲です。 `/m` オプションの値を決める際にはこの値が参考になります。

```
WMDRV Wacom serial tablet and bus mouse driver version *.** (****/**/**)
型番: GD-0608-R00,V1.2-7
座標範囲: (x,y)=(20320,16240)
筆圧閾値: 100, 余白幅: (上,下,左,右)=(80,80,80,80)
カーソル同期: 有効, 独自ファンクション: 無効
常駐しました
```

上記の余白幅の設定では、縦方向を例にとると、ペンタブレット上でのペンの座標が 0～80 の場合はカーソル位置が画面最上部に、ペンの座標が 16160～16240 の場合はカーソル位置が画面最下部になるように動作します。


## ドライバ制御割込

本ソフトウェアを常駐させると、`int 33h` でマウスドライバの機能を呼び出せます。
この割込のインターフェースはマイクロソフト仕様に合わせて実装してあります。

実装済みのファンクションは ax=00h, 01h, 02h, 03h, 04h, 05h, 06h, 07h, 08h, 09h, 0bh, 0ch, 0fh です。

また、独自の(ほぼデバッグ用の)ファンクションとして以下3種類があります。
これらのファンクションは `/d` オプションを付けて常駐すると利用可能になります。

- `int 33h, ax=c0h`
    - タブレットから受信したデータを取得します
    - 戻値: (`cx`,`dx`)=(ペン X 座標, ペン Y 座標), `bx`=筆圧
- `int 33h, ax=c1h`
    - マウスから受信したデータを取得します
    - 戻値: (`cx`,`dx`)=(X 方向ミッキー値, Y 方向ミッキー値), `bx`=ボタン押下状態
- `int 33h, ax=c2h`
    - カーソル表示カウンタの値を取得します
    - 戻値: `dl`=カウンタの値

ドライバの常駐の有無は、(`int 33h` の割込ベクタアドレス+3)バイトの位置から「WMDRV」の NULL 終端文字列があるかどうかで確認できます。


## 使用上の注意及び制約

本ソフトウェアには、使用する際に気を付けなくてはならない事柄や、動作における制約がいくつか存在します。

本プログラムは、ペンタブレットの電源投入後にペンタブレットとの間で一切通信が行われていない状態において、動作確認をしています。
事前にペンタブレットを制御するプログラムを使用する等してペンタブレットとの通信が行われた後に本ソフトウェアを実行すると正しく動作しない可能性があります。

本プログラムで利用可能なタブレットの座標モードは絶対座標のみです。

マウスの割込周期は 120Hz 固定です。


## 作者・更新履歴

あめさろだ(rainy-srd, r-srd), mail: r-srd@outlook.jp

- 2025-01-05, version 0.5a
    - RSA-98III 対応
        - RSA-98II 用ルーチンで動いたため関係する実装に変更はありません
    - 独自ファンクションをデフォルトで利用不可として `/d` オプション追加
    - タブレットとマウスのカーソル位置を同期しないモードを追加
        - `/s` オプションで設定可能
- 2024-07-28, version 0.4a
    - RSA-98II 対応
    - XD シリーズ対応
    - プログラムオプション変更
        - `/c` オプションのカンマに続く IO ポート指定廃止
        - `/a` `/i` `/k` オプション追加
    - 8086 命令のみ使用するように変更
    - ミッキー値の扱いが間違っていたのを修正
    - int 33h (ax=0000h) で DF を破壊していたのを修正
- 2023-10-29, version 0.3a
    - KT シリーズ対応
- 2022-10-02, version 0.2b
    - デフォルトで表示される内蔵のマウスカーソルの形状を更新
- 2022-08-13, version 0.2a
    - PC-9861K 互換の第2,3回線と 2ndCCU に対応
- 2022-07-24, version 0.1b
    - 常駐時に DSR を見て通信相手がいるかチェックする処理を追加
- 2022-07-24, version 0.1a
    - 初版
